// RP6 CCPRO Expansion Selftest
//------------------------------------------------------------------------------
// Dieses kleine Programm berprft ob die onboard Komponenten des CCPRO
// Moduls korrekt funktionieren.
// Wenn Sie dieses Programm aufhren, mssen Sie auf die Ausgaben des
// Programms achten (ber serielle Schnittstelle, auf dem LCD wird nur der
// Starttext angezeigt)
// Achten Sie darauf, dass alle LEDs aufleuchten und keine dunkel bleibt
// und darauf, dass der Piezo Tongeber ab und zu piepstne erzeugt.
//
// Der Rest kann weitgehend automatisch berprft werden. Bei Fehlern
// wird eine entsprechende Meldung ausgegeben.
//
// berprft werden:
// 1. LCD + Starttaster (Begrungsbildschirm wird angezeigt und Sie mssen
// auf den Starttaster drcken um das Programm zu starten!)
// 2. Beeper
// 3. LEDs (Lauflicht)
// 4. I2C Temperatursensor (bitte sorgen Sie dafr, dass die Umgebungstemperatur
// des Roboters zwischen 10 und 40C liegt - am besten zwischen
// 15 und 30C )
// 5. Externer Speicher (64K SRAM) - es werden ein paar Testmuster geschrieben
// und ausgelesen. Fehler werden automatisch erkannt.
// 6. I2C Bus Kommunikation mit dem Controller auf dem Mainboard.
// DAZU MUSS DAS I2C_SLAVE PROGRAMM IN DEN CONTROLLER AUF DEM MAINBOARD
// GELADEN WERDEN!!! (s. RP6 Anleitung)
//
// --------
// Hinweis: Dieses Programm ist nicht ausfhrlich Kommentiert.
// Schauen Sie sich lieber die ausfhrlich kommentierten Beispiele an um
// das Programmieren des RP6 CCPRO Erweiterungsmoduls zu erlernen!
//
//------------------------------------------------------------------------------

#include "../../RP6CCLib/RP6CCLib.cc"

#define RP6_BASE_ADR 10
#define TCN75_ADR 0x90

void error(void)
{
    newline();
    println("############################################################");
    println("SELFTEST FAILED!  PLEASE CHECK MODULE FOR ASSEMBLY MISTAKES!");
    println("############################################################");
    showScreenLCD("SELFTEST FAILED!","x-x-x-x-x-x-x-x-");
    beep(500,250,250);
    beep(500,250,250);
    beep(500,250,250);
    beep(500,250,250);
    while(true)
    {
        setLEDs(LED1 | LED2 | LED3 | LED4 | LED5);
        AbsDelay(250);
        setLEDs(0);
        AbsDelay(300);
    }
}

byte memtest[63000];

void main(void)
{
	RP6_CCPRO_Init();
    setLEDs(LED1 | LED2 | LED3 | LED4 | LED5);

    newline();
    println("   ________________________");
    println("   \\| RP6  ROBOT SYSTEM |/");
    println("    \\_-_-_-_-_-_-_-_-_-_/ ");
    newline();
    println("  RP6 C-Control PRO M128 Selftest  ");
    newline();
    newline();
    println("  Writing Text to LCD...  ");
    newline();
    printLCD("RP6 CCPRO M128");
    setCursorPosLCD(1,0);
    printLCD("SELFTEST v.1.0");
    AbsDelay(1000);
    newline();
    println("  Testing Beeper...  ");

    beep(100,50,50);
    beep(200,100,100);
    beep(300,150,150);
    beep(400,200,200);
    beep(500,250,250);

    setLEDs(0);

    newline();
    println("  Testing LEDs...  ");
    newline();

    byte leds;
    word i;
    leds = 1;
    for(i = 0; i < 26; i++) {
        setLEDs(leds);
        leds = leds << 1;
        if(leds > 17)
            leds = 1;
        Thread_Delay(10);
    }

    beep(200,100,100);

    newline();
    println("  Testing Temperature Sensor...  ");
    newline();

    char result[64];

    for(i = 0; i < 16; i++) {
        TCN75_run(TCN75_ADR, TCN75A_CONFIG_RES_12);
        Thread_Delay(25);
        TCN75_shutdown(TCN75_ADR);
        TCN75_read(TCN75_ADR);
        float temperature;
        int temp_low;

        temp_low = getTemperatureLow();
        if(!temp_low & 128)
            temp_low = (temp_low & 63) - 127;
        else
            temp_low = temp_low & 63;
        temperature = temp_low + (0.0625 * (getTemperatureHigh()>>4));

        print("Temperature: ");
        Str_WriteFloat(temperature,4,result,0);
        Serial_WriteText(0,result);
        print("C");

        if(temperature > 10 && temperature < 40) {
            println(" --> OK");
        }
        else {
            println(" --> OUT OF RANGE!");
            println("TEMPERATURE SENSOR MAY BE DEFECT!");
            error();
        }

        Thread_Delay(25);
    }

    beep(200,100,100);

    byte pattern;
	char pass_fail;
	char tmp[101];
    char tmp2[101];

    newline();
    println("  Testing External 64K Memory...  ");
    word j,k,l;
    pass_fail = 0;
    pattern = 0x11;
    for(i = 0; i < 8; i++) {
        tmp = "MEM PATTERN: ";
        Str_Copy(result,tmp,0);
        Str_WriteInt(pattern,result,STR_APPEND);
        Serial_WriteText(0,result);

        println(" - Writing...");

        Str_Fill(tmp, pattern, 100);
        for(j = 0; j < 630; j++) {
              Str_Copy(memtest, tmp, j*100);
        }
        println("Reading...");
        for(j = 0; j < 63000; j = j + 5) {
            if(memtest[j] != pattern) {
                tmp = "MEMTEST FAIL: ";
                Str_Copy(result,tmp,0);
                Str_WriteInt(j,result,STR_APPEND);
                tmp = " !!!";
                Str_Copy(result,tmp,STR_APPEND);
                Serial_WriteText(0,result);
                pass_fail = -1;
            }
        }
        println("OK");
        pattern = pattern << 1;
    }
    if(pass_fail == 0) {
        println("MEMTEST PASSED!");
        Thread_Delay(200);
    }
    else {
        println("######## MEMTEST FAILED! ########");
        Thread_Delay(400);
    }

    beep(200,100,100);

    newline();
    println("  Testing I2C Communication...  ");
    newline();

    byte led_test;
    leds = 1;
    for(i = 0; i < 25; i++) {
        RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, leds);
        Thread_Delay(10);
        led_test = RP6_readRegister(RP6_BASE_ADR, I2C_REG_LEDS);
        if(led_test != leds) {
            println("I2C COMMUNICATION ERROR!");
            error();
        }
     	leds = leds << 1;
        if(leds > 63)
            leds = 1;
        Thread_Delay(10);
    }

    RP6_writeCMD_1param(RP6_BASE_ADR, CMD_SETLEDS, 0x09);
    Thread_Delay(10);
    led_test = RP6_readRegister(RP6_BASE_ADR, I2C_REG_LEDS);
    if(led_test != 0x09) {
        println("I2C COMMUNICATION ERROR!");
        error();
    }


    newline();
    println("##########################################################");
    println("  Selftest finished successfully!  ");
    newline();

    showScreenLCD("SELFTEST OK!    ","+ + + + + + + + ");

    beep(200,100,100);
    beep(100,50,50);

    while(true) {
        setLEDs(leds);
        leds = leds << 1;
        if(leds > 17)
            leds = 1;
        Thread_Delay(10);
    }
}


